@@ -21,6 +21,7 @@
 #include <wlr/types/wlr_pointer_constraints_v1.h>
 #include <wlr/types/wlr_primary_selection_v1.h>
 #include <wlr/types/wlr_server_decoration.h>
+#include <wlr/types/wlr_single_pixel_buffer_v1.h>
 #include <wlr/types/wlr_tablet_v2.h>
 #include <wlr/types/wlr_viewporter.h>
 #include <wlr/types/wlr_xcursor_manager.h>
@@ -33,6 +34,7 @@
 #include <wlr/util/box.h>
 #include <wlr/version.h>
 #include "cursor.h"
+#include "device-state.h"
 #include "idle-inhibit.h"
 #include "layers.h"
 #include "output.h"
@@ -66,12 +68,19 @@ static GParamSpec *props[PROP_LAST_PROP];
 
 
 typedef struct _PhocDesktopPrivate {
-  PhocIdleInhibit *idle_inhibit;
+  PhocIdleInhibit       *idle_inhibit;
 
-  gboolean         enable_animations;
+  gboolean               enable_animations;
 
-  GSettings       *settings;
-  GSettings       *interface_settings;
+  GSettings             *settings;
+  GSettings             *interface_settings;
+
+  /* Protocols without upstreamable implementations */
+  PhocPhoshPrivate      *phosh;
+  PhocGtkShell          *gtk_shell;
+  /* Protocols that should go upstream */
+  PhocLayerShellEffects *layer_shell_effects;
+  PhocDeviceState       *device_state;
 } PhocDesktopPrivate;
 
 G_DEFINE_TYPE_WITH_PRIVATE (PhocDesktop, phoc_desktop, G_TYPE_OBJECT);
@@ -242,72 +251,70 @@ layer_surface_at (PhocOutput                     *output,
  *
  * Returns: (nullable): The `struct wlr_surface`
  */
-struct wlr_surface *phoc_desktop_surface_at(PhocDesktop *desktop,
-		double lx, double ly, double *sx, double *sy,
-		PhocView **view) {
-	struct wlr_surface *surface = NULL;
-	struct wlr_output *wlr_output =
-		wlr_output_layout_output_at(desktop->layout, lx, ly);
-	PhocOutput *phoc_output = NULL;
-	double ox = lx, oy = ly;
-	if (view) {
-		*view = NULL;
-	}
+struct wlr_surface *
+phoc_desktop_surface_at(PhocDesktop *desktop,
+                        double lx, double ly, double *sx, double *sy,
+                        PhocView **view)
+{
+  struct wlr_surface *surface = NULL;
+  PhocOutput *output = phoc_desktop_layout_get_output (desktop, lx, ly);
+  double ox = lx, oy = ly;
+  if (view) {
+    *view = NULL;
+  }
 
-	if (wlr_output) {
-		phoc_output = PHOC_OUTPUT (wlr_output->data);
-		wlr_output_layout_output_coords(desktop->layout, wlr_output, &ox, &oy);
+  if (output) {
+    wlr_output_layout_output_coords(desktop->layout, output->wlr_output, &ox, &oy);
 
-		if ((surface = layer_surface_at(phoc_output, ZWLR_LAYER_SHELL_V1_LAYER_OVERLAY,
-						ox, oy, sx, sy))) {
-			return surface;
-		}
+    if ((surface = layer_surface_at(output, ZWLR_LAYER_SHELL_V1_LAYER_OVERLAY,
+                                    ox, oy, sx, sy))) {
+      return surface;
+    }
 
-		PhocOutput *output = wlr_output->data;
-		if (output != NULL && output->fullscreen_view != NULL) {
+    if (output->fullscreen_view != NULL) {
 
-			if (phoc_output_has_shell_revealed (output)) {
-				if ((surface = layer_surface_at(phoc_output, ZWLR_LAYER_SHELL_V1_LAYER_TOP,
-								ox, oy, sx, sy))) {
-					return surface;
-				}
-			}
+      if (phoc_output_has_shell_revealed (output)) {
+        if ((surface = layer_surface_at(output, ZWLR_LAYER_SHELL_V1_LAYER_TOP,
+                                        ox, oy, sx, sy))) {
+          return surface;
+        }
+      }
 
-			if (view_at(output->fullscreen_view, lx, ly, &surface, sx, sy)) {
-				if (view) {
-					*view = output->fullscreen_view;
-				}
-				return surface;
-			} else {
-				return NULL;
-			}
-		}
+      if (view_at(output->fullscreen_view, lx, ly, &surface, sx, sy)) {
+        if (view) {
+          *view = output->fullscreen_view;
+        }
+        return surface;
+      } else {
+        return NULL;
+      }
+    }
 
-		if ((surface = layer_surface_at(phoc_output, ZWLR_LAYER_SHELL_V1_LAYER_TOP,
-						ox, oy, sx, sy))) {
-			return surface;
-		}
-	}
+    if ((surface = layer_surface_at(output, ZWLR_LAYER_SHELL_V1_LAYER_TOP,
+                                    ox, oy, sx, sy))) {
+      return surface;
+    }
+  }
 
-	PhocView *_view;
-	if ((_view = desktop_view_at(desktop, lx, ly, &surface, sx, sy))) {
-		if (view) {
-			*view = _view;
-		}
-		return surface;
-	}
+  PhocView *_view;
+  if ((_view = desktop_view_at(desktop, lx, ly, &surface, sx, sy))) {
+    if (view) {
+      *view = _view;
+    }
+    return surface;
+  }
 
-	if (wlr_output) {
-		if ((surface = layer_surface_at(phoc_output, ZWLR_LAYER_SHELL_V1_LAYER_BOTTOM,
-						ox, oy, sx, sy))) {
-			return surface;
-		}
-		if ((surface = layer_surface_at(phoc_output, ZWLR_LAYER_SHELL_V1_LAYER_BACKGROUND,
-						ox, oy, sx, sy))) {
-			return surface;
-		}
-	}
-	return NULL;
+  if (output) {
+    if ((surface = layer_surface_at(output, ZWLR_LAYER_SHELL_V1_LAYER_BOTTOM,
+                                    ox, oy, sx, sy))) {
+      return surface;
+    }
+    if ((surface = layer_surface_at(output, ZWLR_LAYER_SHELL_V1_LAYER_BACKGROUND,
+                                    ox, oy, sx, sy))) {
+      return surface;
+    }
+  }
+  return NULL;
 }
 
 gboolean
@@ -358,7 +365,7 @@ handle_layout_change (struct wl_listener *listener, void *data)
 {
   PhocDesktop *self;
   struct wlr_output *center_output;
-  struct wlr_box *center_output_box;
+  struct wlr_box center_output_box;
   double center_x, center_y;
   PhocView *view;
   PhocOutput *output;
@@ -368,9 +375,9 @@ handle_layout_change (struct wl_listener *listener, void *data)
   if (center_output == NULL)
     return;
 
-  center_output_box = wlr_output_layout_get_box (self->layout, center_output);
-  center_x = center_output_box->x + center_output_box->width / 2;
-  center_y = center_output_box->y + center_output_box->height / 2;
+  wlr_output_layout_get_box (self->layout, center_output, &center_output_box);
+  center_x = center_output_box.x + center_output_box.width / 2;
+  center_y = center_output_box.y + center_output_box.height / 2;
 
   /* Make sure all views are on an existing output */
   wl_list_for_each (view, &self->views, link) {
@@ -387,89 +394,95 @@ handle_layout_change (struct wl_listener *listener, void *data)
     phoc_output_damage_whole(output);
 }
 
-static void input_inhibit_activate(struct wl_listener *listener, void *data) {
-	PhocDesktop *desktop = wl_container_of(
-			listener, desktop, input_inhibit_activate);
-	PhocServer *server = phoc_server_get_default ();
 
-	for (GSList *elem = phoc_input_get_seats (server->input); elem; elem = elem->next) {
-		PhocSeat *seat = PHOC_SEAT (elem->data);
+static void
+input_inhibit_activate (struct wl_listener *listener, void *data)
+{
+  PhocDesktop *desktop = wl_container_of(listener, desktop, input_inhibit_activate);
+  PhocServer *server = phoc_server_get_default ();
 
-		g_assert (PHOC_IS_SEAT (seat));
-		phoc_seat_set_exclusive_client(seat,
-				desktop->input_inhibit->active_client);
-	}
+  for (GSList *elem = phoc_input_get_seats (server->input); elem; elem = elem->next) {
+    PhocSeat *seat = PHOC_SEAT (elem->data);
+
+    g_assert (PHOC_IS_SEAT (seat));
+    phoc_seat_set_exclusive_client (seat, desktop->input_inhibit->active_client);
+  }
 }
 
-static void input_inhibit_deactivate(struct wl_listener *listener, void *data) {
-	PhocServer *server = phoc_server_get_default ();
 
-	for (GSList *elem = phoc_input_get_seats (server->input); elem; elem = elem->next) {
-		PhocSeat *seat = PHOC_SEAT (elem->data);
+static void
+input_inhibit_deactivate (struct wl_listener *listener, void *data)
+{
+  PhocServer *server = phoc_server_get_default ();
 
-		g_assert (PHOC_IS_SEAT (seat));
-		phoc_seat_set_exclusive_client(seat, NULL);
-	}
+  for (GSList *elem = phoc_input_get_seats (server->input); elem; elem = elem->next) {
+    PhocSeat *seat = PHOC_SEAT (elem->data);
+
+    g_assert (PHOC_IS_SEAT (seat));
+    phoc_seat_set_exclusive_client (seat, NULL);
+  }
 }
 
-static void handle_constraint_destroy(struct wl_listener *listener,
-		void *data) {
-	PhocPointerConstraint *constraint =
-		wl_container_of(listener, constraint, destroy);
-	struct wlr_pointer_constraint_v1 *wlr_constraint = data;
-	PhocSeat *seat = wlr_constraint->seat->data;
-	PhocCursor *cursor = phoc_seat_get_cursor (seat);
 
-	wl_list_remove(&constraint->destroy.link);
+static void
+handle_constraint_destroy (struct wl_listener *listener, void *data)
+{
+  PhocPointerConstraint *constraint = wl_container_of(listener, constraint, destroy);
+  struct wlr_pointer_constraint_v1 *wlr_constraint = data;
+  PhocSeat *seat = wlr_constraint->seat->data;
+  PhocCursor *cursor = phoc_seat_get_cursor (seat);
 
-	if (cursor->active_constraint == wlr_constraint) {
-		wl_list_remove(&cursor->constraint_commit.link);
-		wl_list_init(&cursor->constraint_commit.link);
-		cursor->active_constraint = NULL;
+  wl_list_remove(&constraint->destroy.link);
 
-		if (wlr_constraint->current.committed &
-				WLR_POINTER_CONSTRAINT_V1_STATE_CURSOR_HINT &&
-				cursor->pointer_view) {
-			PhocView *view = cursor->pointer_view->view;
-			double lx = view->box.x + wlr_constraint->current.cursor_hint.x;
-			double ly = view->box.y + wlr_constraint->current.cursor_hint.y;
+  if (cursor->active_constraint == wlr_constraint) {
+    wl_list_remove(&cursor->constraint_commit.link);
+    wl_list_init(&cursor->constraint_commit.link);
+    cursor->active_constraint = NULL;
 
-			wlr_cursor_warp(cursor->cursor, NULL, lx, ly);
-		}
-	}
+    if (wlr_constraint->current.committed &
+        WLR_POINTER_CONSTRAINT_V1_STATE_CURSOR_HINT &&
+        cursor->pointer_view) {
+      PhocView *view = cursor->pointer_view->view;
+      double lx = view->box.x + wlr_constraint->current.cursor_hint.x;
+      double ly = view->box.y + wlr_constraint->current.cursor_hint.y;
 
-	free(constraint);
+      wlr_cursor_warp(cursor->cursor, NULL, lx, ly);
+    }
+  }
+
+  free(constraint);
 }
 
-static void handle_pointer_constraint(struct wl_listener *listener,
-		void *data) {
-	PhocServer *server = phoc_server_get_default ();
-	struct wlr_pointer_constraint_v1 *wlr_constraint = data;
-	PhocSeat *seat = wlr_constraint->seat->data;
-	PhocCursor *cursor = phoc_seat_get_cursor (seat);
+static
+void handle_pointer_constraint (struct wl_listener *listener, void *data)
+{
+  PhocServer *server = phoc_server_get_default ();
+  struct wlr_pointer_constraint_v1 *wlr_constraint = data;
+  PhocSeat *seat = wlr_constraint->seat->data;
+  PhocCursor *cursor = phoc_seat_get_cursor (seat);
 
-	PhocPointerConstraint *constraint =
-		calloc(1, sizeof(PhocPointerConstraint));
-	constraint->constraint = wlr_constraint;
+  PhocPointerConstraint *constraint =
+    calloc(1, sizeof(PhocPointerConstraint));
+  constraint->constraint = wlr_constraint;
 
-	constraint->destroy.notify = handle_constraint_destroy;
-	wl_signal_add(&wlr_constraint->events.destroy, &constraint->destroy);
+  constraint->destroy.notify = handle_constraint_destroy;
+  wl_signal_add(&wlr_constraint->events.destroy, &constraint->destroy);
 
-	double sx, sy;
-	struct wlr_surface *surface = phoc_desktop_surface_at(
-		server->desktop,
-		cursor->cursor->x, cursor->cursor->y, &sx, &sy, NULL);
+  double sx, sy;
+  struct wlr_surface *surface = phoc_desktop_surface_at(
+    server->desktop,
+    cursor->cursor->x, cursor->cursor->y, &sx, &sy, NULL);
 
-	if (surface == wlr_constraint->surface) {
-		assert(!cursor->active_constraint);
-		phoc_cursor_constrain(cursor, wlr_constraint, sx, sy);
-	}
+  if (surface == wlr_constraint->surface) {
+    g_assert (!cursor->active_constraint);
+    phoc_cursor_constrain(cursor, wlr_constraint, sx, sy);
+  }
 }
 
 static void
 auto_maximize_changed_cb (PhocDesktop *self,
-			  const gchar *key,
-			  GSettings   *settings)
+                          const gchar *key,
+                          GSettings   *settings)
 {
   gboolean max = g_settings_get_boolean (settings, key);
 
@@ -498,14 +511,15 @@ on_enable_animations_changed (PhocDesktop *self,
 
 #ifdef PHOC_XWAYLAND
 static const char *atom_map[XWAYLAND_ATOM_LAST] = {
-	"_NET_WM_WINDOW_TYPE_NORMAL",
-	"_NET_WM_WINDOW_TYPE_DIALOG"
+        "_NET_WM_WINDOW_TYPE_NORMAL",
+        "_NET_WM_WINDOW_TYPE_DIALOG"
 };
 
-static
-void handle_xwayland_ready(struct wl_listener *listener, void *data) {
-  PhocDesktop *desktop = wl_container_of (
-        listener, desktop, xwayland_ready);
+static void
+handle_xwayland_ready (struct wl_listener *listener,
+                       void               *data)
+{
+  PhocDesktop *desktop = wl_container_of (listener, desktop, xwayland_ready);
   xcb_connection_t *xcb_conn = xcb_connect (NULL, NULL);
 
   int err = xcb_connection_has_error (xcb_conn);
@@ -536,18 +550,28 @@ void handle_xwayland_ready(struct wl_listener *listener, void *data) {
   }
 
   xcb_disconnect (xcb_conn);
+
+#ifdef PHOC_XWAYLAND
+  if (desktop->xwayland != NULL) {
+    PhocSeat *xwayland_seat = phoc_input_get_seat (phoc_server_get_default ()->input,
+                                                   PHOC_CONFIG_DEFAULT_SEAT_NAME);
+    wlr_xwayland_set_seat (desktop->xwayland, xwayland_seat->seat);
+  }
+#endif
+
 }
 
-static
-void handle_xwayland_remove_startup_id(struct wl_listener *listener, void *data) {
-  PhocDesktop *desktop = wl_container_of (
-        listener, desktop, xwayland_remove_startup_id);
+
+static void
+handle_xwayland_remove_startup_id (struct wl_listener *listener, void *data)
+{
+  PhocDesktop *desktop = wl_container_of (listener, desktop, xwayland_remove_startup_id);
   struct wlr_xwayland_remove_startup_info_event *ev = data;
 
   g_assert (PHOC_IS_DESKTOP (desktop));
   g_assert (ev->id);
 
-  phoc_phosh_private_notify_startup_id (desktop->phosh,
+  phoc_phosh_private_notify_startup_id (phoc_desktop_get_phosh_private (desktop),
                                         ev->id,
                                         PHOSH_PRIVATE_STARTUP_TRACKER_PROTOCOL_X11);
 }
@@ -623,7 +647,7 @@ phoc_desktop_setup_xwayland (PhocDesktop *self)
 
   if (config->xwayland) {
     self->xwayland = wlr_xwayland_create(server->wl_display,
-					 server->compositor, config->xwayland_lazy);
+                                         server->compositor, config->xwayland_lazy);
     if (!self->xwayland) {
       g_critical ("Failed to initialize Xwayland");
       g_unsetenv ("DISPLAY");
@@ -631,15 +655,15 @@ phoc_desktop_setup_xwayland (PhocDesktop *self)
     }
 
     wl_signal_add(&self->xwayland->events.new_surface,
-		  &self->xwayland_surface);
+                  &self->xwayland_surface);
     self->xwayland_surface.notify = handle_xwayland_surface;
 
     wl_signal_add(&self->xwayland->events.ready,
-		  &self->xwayland_ready);
+                  &self->xwayland_ready);
     self->xwayland_ready.notify = handle_xwayland_ready;
 
     wl_signal_add(&self->xwayland->events.remove_startup_info,
-		  &self->xwayland_remove_startup_id);
+                  &self->xwayland_remove_startup_id);
     self->xwayland_remove_startup_id.notify = handle_xwayland_remove_startup_id;
 
     g_setenv ("DISPLAY", self->xwayland->display_name, true);
@@ -648,12 +672,12 @@ phoc_desktop_setup_xwayland (PhocDesktop *self)
       g_critical ("Cannot load XWayland XCursor theme");
 
     struct wlr_xcursor *xcursor = wlr_xcursor_manager_get_xcursor(
-								  self->xcursor_manager, cursor_default, 1);
+                                                                  self->xcursor_manager, cursor_default, 1);
     if (xcursor != NULL) {
       struct wlr_xcursor_image *image = xcursor->images[0];
       wlr_xwayland_set_cursor(self->xwayland, image->buffer,
-			      image->width * 4, image->width, image->height, image->hotspot_x,
-			      image->hotspot_y);
+                              image->width * 4, image->width, image->height, image->hotspot_x,
+                              image->hotspot_y);
     }
   }
 #endif
@@ -669,127 +693,107 @@ phoc_desktop_constructed (GObject *object)
 
   G_OBJECT_CLASS (phoc_desktop_parent_class)->constructed (object);
 
-  wl_list_init(&self->views);
-  wl_list_init(&self->outputs);
+  wl_list_init (&self->views);
+  wl_list_init (&self->outputs);
 
   self->new_output.notify = handle_new_output;
-  wl_signal_add(&server->backend->events.new_output, &self->new_output);
+  wl_signal_add (&server->backend->events.new_output, &self->new_output);
 
-  self->layout = wlr_output_layout_create();
-  wlr_xdg_output_manager_v1_create(server->wl_display, self->layout);
+  self->layout = wlr_output_layout_create ();
+  wlr_xdg_output_manager_v1_create (server->wl_display, self->layout);
   self->layout_change.notify = handle_layout_change;
-  wl_signal_add(&self->layout->events.change, &self->layout_change);
+  wl_signal_add (&self->layout->events.change, &self->layout_change);
 
-  self->xdg_shell = wlr_xdg_shell_create(server->wl_display);
-  wl_signal_add(&self->xdg_shell->events.new_surface,
-		&self->xdg_shell_surface);
+  self->xdg_shell = wlr_xdg_shell_create (server->wl_display, 5);
+  wl_signal_add (&self->xdg_shell->events.new_surface, &self->xdg_shell_surface);
   self->xdg_shell_surface.notify = handle_xdg_shell_surface;
 
-  self->layer_shell = wlr_layer_shell_v1_create(server->wl_display);
-  wl_signal_add(&self->layer_shell->events.new_surface,
-		&self->layer_shell_surface);
+  self->layer_shell = wlr_layer_shell_v1_create (server->wl_display);
+  wl_signal_add (&self->layer_shell->events.new_surface, &self->layer_shell_surface);
   self->layer_shell_surface.notify = handle_layer_shell_surface;
-  self->layer_shell_effects = phoc_layer_shell_effects_new ();
+  priv->layer_shell_effects = phoc_layer_shell_effects_new ();
 
-  self->tablet_v2 = wlr_tablet_v2_create(server->wl_display);
+  self->tablet_v2 = wlr_tablet_v2_create (server->wl_display);
 
   char cursor_size_fmt[16];
-  snprintf(cursor_size_fmt, sizeof(cursor_size_fmt),
-	   "%d", PHOC_XCURSOR_SIZE);
-  g_setenv("XCURSOR_SIZE", cursor_size_fmt, 1);
+  snprintf (cursor_size_fmt, sizeof (cursor_size_fmt), "%d", PHOC_XCURSOR_SIZE);
+  g_setenv ("XCURSOR_SIZE", cursor_size_fmt, 1);
 
   phoc_desktop_setup_xwayland (self);
 
-  self->gamma_control_manager_v1 = wlr_gamma_control_manager_v1_create(server->wl_display);
-  self->export_dmabuf_manager_v1 =
-    wlr_export_dmabuf_manager_v1_create(server->wl_display);
-  self->server_decoration_manager =
-    wlr_server_decoration_manager_create(server->wl_display);
-  wlr_server_decoration_manager_set_default_mode(self->server_decoration_manager,
-						 WLR_SERVER_DECORATION_MANAGER_MODE_CLIENT);
-  self->idle = wlr_idle_create(server->wl_display);
+  self->gamma_control_manager_v1 = wlr_gamma_control_manager_v1_create (server->wl_display);
+  self->export_dmabuf_manager_v1 = wlr_export_dmabuf_manager_v1_create (server->wl_display);
+  self->server_decoration_manager = wlr_server_decoration_manager_create (server->wl_display);
+  wlr_server_decoration_manager_set_default_mode (self->server_decoration_manager,
+                                                  WLR_SERVER_DECORATION_MANAGER_MODE_CLIENT);
+  self->idle = wlr_idle_create (server->wl_display);
   self->primary_selection_device_manager =
-    wlr_primary_selection_v1_device_manager_create(server->wl_display);
-  self->input_inhibit =
-    wlr_input_inhibit_manager_create(server->wl_display);
+    wlr_primary_selection_v1_device_manager_create (server->wl_display);
+
+  self->input_inhibit = wlr_input_inhibit_manager_create (server->wl_display);
   self->input_inhibit_activate.notify = input_inhibit_activate;
-  wl_signal_add(&self->input_inhibit->events.activate,
-		&self->input_inhibit_activate);
+  wl_signal_add (&self->input_inhibit->events.activate, &self->input_inhibit_activate);
   self->input_inhibit_deactivate.notify = input_inhibit_deactivate;
-  wl_signal_add(&self->input_inhibit->events.deactivate,
-		&self->input_inhibit_deactivate);
+  wl_signal_add (&self->input_inhibit->events.deactivate, &self->input_inhibit_deactivate);
 
-  self->input_method =
-    wlr_input_method_manager_v2_create(server->wl_display);
-
-  self->text_input = wlr_text_input_manager_v3_create(server->wl_display);
+  self->input_method = wlr_input_method_manager_v2_create (server->wl_display);
+  self->text_input = wlr_text_input_manager_v3_create (server->wl_display);
 
   priv->idle_inhibit = phoc_idle_inhibit_create (self->idle);
-  self->gtk_shell = phoc_gtk_shell_create(self, server->wl_display);
-  self->phosh = phoc_phosh_private_new ();
+
+  priv->gtk_shell = phoc_gtk_shell_create (self, server->wl_display);
+  priv->phosh = phoc_phosh_private_new ();
 
   self->xdg_activation_v1 = wlr_xdg_activation_v1_create (server->wl_display);
   self->xdg_activation_v1_request_activate.notify = phoc_xdg_activation_v1_handle_request_activate;
   wl_signal_add (&self->xdg_activation_v1->events.request_activate,
                  &self->xdg_activation_v1_request_activate);
 
-  self->virtual_keyboard = wlr_virtual_keyboard_manager_v1_create(
-								  server->wl_display);
-  wl_signal_add(&self->virtual_keyboard->events.new_virtual_keyboard,
-		&self->virtual_keyboard_new);
+  self->virtual_keyboard = wlr_virtual_keyboard_manager_v1_create (server->wl_display);
+  wl_signal_add (&self->virtual_keyboard->events.new_virtual_keyboard,
+                 &self->virtual_keyboard_new);
   self->virtual_keyboard_new.notify = phoc_handle_virtual_keyboard;
 
-  self->virtual_pointer = wlr_virtual_pointer_manager_v1_create(server->wl_display);
-  wl_signal_add(&self->virtual_pointer->events.new_virtual_pointer,
-		&self->virtual_pointer_new);
+  self->virtual_pointer = wlr_virtual_pointer_manager_v1_create (server->wl_display);
+  wl_signal_add (&self->virtual_pointer->events.new_virtual_pointer, &self->virtual_pointer_new);
   self->virtual_pointer_new.notify = phoc_handle_virtual_pointer;
 
-  self->screencopy = wlr_screencopy_manager_v1_create(server->wl_display);
+  self->screencopy = wlr_screencopy_manager_v1_create (server->wl_display);
+
+  self->xdg_decoration_manager = wlr_xdg_decoration_manager_v1_create (server->wl_display);
+  wl_signal_add (&self->xdg_decoration_manager->events.new_toplevel_decoration,
+                 &self->xdg_toplevel_decoration);
 
-  self->xdg_decoration_manager =
-    wlr_xdg_decoration_manager_v1_create(server->wl_display);
-  wl_signal_add(&self->xdg_decoration_manager->events.new_toplevel_decoration,
-		&self->xdg_toplevel_decoration);
   self->xdg_toplevel_decoration.notify = handle_xdg_toplevel_decoration;
-  wlr_viewporter_create(server->wl_display);
+  wlr_viewporter_create (server->wl_display);
+  wlr_single_pixel_buffer_manager_v1_create (server->wl_display);
 
   struct wlr_xdg_foreign_registry *foreign_registry =
-                wlr_xdg_foreign_registry_create(server->wl_display);
-  wlr_xdg_foreign_v1_create(server->wl_display, foreign_registry);
-  wlr_xdg_foreign_v2_create(server->wl_display, foreign_registry);
+                wlr_xdg_foreign_registry_create (server->wl_display);
+  wlr_xdg_foreign_v1_create (server->wl_display, foreign_registry);
+  wlr_xdg_foreign_v2_create (server->wl_display, foreign_registry);
 
-  self->pointer_constraints =
-    wlr_pointer_constraints_v1_create(server->wl_display);
+  self->pointer_constraints = wlr_pointer_constraints_v1_create (server->wl_display);
   self->pointer_constraint.notify = handle_pointer_constraint;
-  wl_signal_add(&self->pointer_constraints->events.new_constraint,
-		&self->pointer_constraint);
+  wl_signal_add (&self->pointer_constraints->events.new_constraint, &self->pointer_constraint);
 
-  self->presentation =
-    wlr_presentation_create(server->wl_display, server->backend);
-  self->foreign_toplevel_manager_v1 =
-    wlr_foreign_toplevel_manager_v1_create(server->wl_display);
-  self->relative_pointer_manager =
-    wlr_relative_pointer_manager_v1_create(server->wl_display);
-  self->pointer_gestures =
-    wlr_pointer_gestures_v1_create(server->wl_display);
+  self->presentation = wlr_presentation_create (server->wl_display, server->backend);
+  self->foreign_toplevel_manager_v1 = wlr_foreign_toplevel_manager_v1_create (server->wl_display);
+  self->relative_pointer_manager = wlr_relative_pointer_manager_v1_create (server->wl_display);
+  self->pointer_gestures = wlr_pointer_gestures_v1_create (server->wl_display);
 
-  self->output_manager_v1 =
-    wlr_output_manager_v1_create(server->wl_display);
+  self->output_manager_v1 = wlr_output_manager_v1_create (server->wl_display);
   self->output_manager_apply.notify = handle_output_manager_apply;
-  wl_signal_add(&self->output_manager_v1->events.apply,
-		&self->output_manager_apply);
+  wl_signal_add (&self->output_manager_v1->events.apply, &self->output_manager_apply);
   self->output_manager_test.notify = handle_output_manager_test;
-  wl_signal_add(&self->output_manager_v1->events.test,
-		&self->output_manager_test);
+  wl_signal_add (&self->output_manager_v1->events.test, &self->output_manager_test);
 
-  self->output_power_manager_v1 =
-    wlr_output_power_manager_v1_create(server->wl_display);
-  self->output_power_manager_set_mode.notify =
-    phoc_output_handle_output_power_manager_set_mode;
-  wl_signal_add(&self->output_power_manager_v1->events.set_mode,
-		&self->output_power_manager_set_mode);
+  self->output_power_manager_v1 = wlr_output_power_manager_v1_create (server->wl_display);
+  self->output_power_manager_set_mode.notify = phoc_output_handle_output_power_manager_set_mode;
+  wl_signal_add (&self->output_power_manager_v1->events.set_mode,
+                 &self->output_power_manager_set_mode);
 
-  wlr_data_control_manager_v1_create(server->wl_display);
+  wlr_data_control_manager_v1_create (server->wl_display);
 
   /* sm.puri.phosh settings */
   priv->settings = g_settings_new ("sm.puri.phoc");
@@ -847,9 +851,10 @@ phoc_desktop_finalize (GObject *object)
 #endif
 
   g_clear_pointer (&priv->idle_inhibit, phoc_idle_inhibit_destroy);
-  g_clear_object (&self->phosh);
-  g_clear_pointer (&self->gtk_shell, phoc_gtk_shell_destroy);
-  g_clear_object (&self->layer_shell_effects);
+  g_clear_object (&priv->phosh);
+  g_clear_pointer (&priv->gtk_shell, phoc_gtk_shell_destroy);
+  g_clear_object (&priv->layer_shell_effects);
+  g_clear_object (&priv->device_state);
   g_clear_pointer (&self->layout, wlr_output_layout_destroy);
 
   g_hash_table_remove_all (self->input_output_map);
@@ -1135,6 +1140,30 @@ phoc_desktop_get_builtin_output (PhocDesktop *self)
   return NULL;
 }
 
+/**
+ * phoc_desktop_layout_get_output:
+ * @self: The desktop
+ * @lx: The x coordinate in layout coordinates
+ * @ly: The y coordinate in layout coordinates
+ *
+ * Get the output at layout coordinates.
+ *
+ * Returns: (transfer none) (nullable): The output
+ */
+PhocOutput *
+phoc_desktop_layout_get_output (PhocDesktop *self, double lx, double ly)
+{
+  struct wlr_output *wlr_output;
+
+  g_assert (PHOC_IS_DESKTOP (self));
+
+  wlr_output = wlr_output_layout_output_at (self->layout, lx, ly);
+  if (wlr_output == NULL)
+    return NULL;
+
+  return PHOC_OUTPUT (wlr_output->data);
+}
+
 /**
  * phoc_desktop_get_draggable_layer_surface:
  * @self: The `PhocDesktop` to look the surface up for
@@ -1143,14 +1172,57 @@ phoc_desktop_get_builtin_output (PhocDesktop *self)
  * Returns a draggable layer surface if `layer_surface` is configured as
  * such. `NULL` otherwise.
  *
- * Returns: (nullable): The `PhocDraggableLayerSurface`
+ * Returns: (transfer none)(nullable): The `PhocDraggableLayerSurface`
  */
 PhocDraggableLayerSurface *
 phoc_desktop_get_draggable_layer_surface (PhocDesktop *self, PhocLayerSurface *layer_surface)
 {
+  PhocDesktopPrivate *priv;
+
   g_assert (PHOC_IS_DESKTOP (self));
   g_assert (layer_surface);
 
+  priv = phoc_desktop_get_instance_private (self);
   return phoc_layer_shell_effects_get_draggable_layer_surface_from_layer_surface (
-    self->layer_shell_effects, layer_surface);
+    priv->layer_shell_effects, layer_surface);
+}
+
+/**
+ * phoc_desktop_get_gtk_shell:
+ * @self: The `PhocDesktop`
+ *
+ * Gets a handler of the gtk_shell Wayland protocol implementations
+ *
+ * Returns: (transfer none): gtk_shell protocol implementation handler
+ */
+PhocGtkShell *
+phoc_desktop_get_gtk_shell (PhocDesktop *self)
+{
+  PhocDesktopPrivate *priv;
+
+  g_assert (PHOC_IS_DESKTOP (self));
+
+  priv = phoc_desktop_get_instance_private (self);
+
+  return priv->gtk_shell;
+}
+
+/**
+ * phoc_desktop_get_phosh_private:
+ * @self: The `PhocDesktop`
+ *
+ * Gets a handler of the phosh-private Wayland protocol implementations
+ *
+ * Returns: (transfer none): phosh_private protocol implementation handler
+ */
+PhocPhoshPrivate *
+phoc_desktop_get_phosh_private (PhocDesktop *self)
+{
+  PhocDesktopPrivate *priv;
+
+  g_assert (PHOC_IS_DESKTOP (self));
+
+  priv = phoc_desktop_get_instance_private (self);
+
+  return priv->phosh;
 }
