@@ -43,16 +43,16 @@ G_DEFINE_TYPE (PhocKeybindings, phoc_keybindings, G_TYPE_OBJECT);
 static void
 handle_maximize (PhocSeat *seat)
 {
-  PhocView *focus = phoc_seat_get_focus(seat);
+  PhocView *focus = phoc_seat_get_focus_view (seat);
 
   if (focus != NULL)
-    view_maximize(focus, NULL);
+    phoc_view_maximize (focus, NULL);
 }
 
 static void
 handle_unmaximize (PhocSeat *seat)
 {
-  PhocView *focus = phoc_seat_get_focus(seat);
+  PhocView *focus = phoc_seat_get_focus_view (seat);
 
   if (focus != NULL)
     view_restore(focus);
@@ -61,7 +61,7 @@ handle_unmaximize (PhocSeat *seat)
 static void
 handle_tile_right (PhocSeat *seat)
 {
-  PhocView *view = phoc_seat_get_focus(seat);
+  PhocView *view = phoc_seat_get_focus_view (seat);
 
   if (!view)
     return;
@@ -76,7 +76,7 @@ handle_tile_right (PhocSeat *seat)
 static void
 handle_tile_left (PhocSeat *seat)
 {
-  PhocView *view = phoc_seat_get_focus(seat);
+  PhocView *view = phoc_seat_get_focus_view (seat);
 
   if (!view)
     return;
@@ -91,20 +91,20 @@ handle_tile_left (PhocSeat *seat)
 static void
 handle_toggle_maximized (PhocSeat *seat)
 {
-  PhocView *focus = phoc_seat_get_focus(seat);
+  PhocView *focus = phoc_seat_get_focus_view (seat);
 
   if (focus != NULL) {
     if (view_is_maximized(focus))
       view_restore(focus);
     else
-      view_maximize(focus, NULL);
+      phoc_view_maximize (focus, NULL);
   }
 }
 
 static void
 handle_toggle_fullscreen (PhocSeat *seat)
 {
-  PhocView *focus = phoc_seat_get_focus(seat);
+  PhocView *focus = phoc_seat_get_focus_view (seat);
 
   if (focus) {
     phoc_view_set_fullscreen(focus, !view_is_fullscreen (focus), NULL);
@@ -126,16 +126,37 @@ static void handle_cycle_windows_backwards (PhocSeat *seat)
 
 static void handle_close (PhocSeat *seat)
 {
-  PhocView *focus = phoc_seat_get_focus(seat);
+  PhocView *focus = phoc_seat_get_focus_view (seat);
 
   if (focus)
     phoc_view_close (focus);
 }
 
+
+static void
+handle_move_to_monitor_up (PhocSeat *seat)
+{
+  PhocView *view = phoc_seat_get_focus_view (seat);
+
+  if (view)
+    view_move_to_next_output (view, WLR_DIRECTION_UP);
+}
+
+
+static void
+handle_move_to_monitor_down (PhocSeat *seat)
+{
+  PhocView *view = phoc_seat_get_focus_view (seat);
+
+  if (view)
+    view_move_to_next_output (view, WLR_DIRECTION_DOWN);
+}
+
+
 static void
 handle_move_to_monitor_right (PhocSeat *seat)
 {
-  PhocView *view = phoc_seat_get_focus(seat);
+  PhocView *view = phoc_seat_get_focus_view (seat);
 
   if (view)
     view_move_to_next_output(view, WLR_DIRECTION_RIGHT);
@@ -145,7 +166,7 @@ handle_move_to_monitor_right (PhocSeat *seat)
 static void
 handle_move_to_monitor_left (PhocSeat *seat)
 {
-  PhocView *view = phoc_seat_get_focus(seat);
+  PhocView *view = phoc_seat_get_focus_view (seat);
 
   if (view)
     view_move_to_next_output(view, WLR_DIRECTION_LEFT);
@@ -558,6 +579,10 @@ phoc_keybindings_constructed (GObject *object)
                        "toggle-fullscreen", handle_toggle_fullscreen);
   phoc_add_keybinding (self, self->settings,
                        "toggle-maximized", handle_toggle_maximized);
+  phoc_add_keybinding (self, self->settings,
+                       "move-to-monitor-up", handle_move_to_monitor_up);
+  phoc_add_keybinding (self, self->settings,
+                       "move-to-monitor-down", handle_move_to_monitor_down);
   phoc_add_keybinding (self, self->settings,
                        "move-to-monitor-right", handle_move_to_monitor_right);
   phoc_add_keybinding (self, self->settings,
