@@ -16,7 +16,6 @@
 #include <unistd.h>
 #include <wayland-server-core.h>
 #include <wlr/config.h>
-#include <wlr/types/wlr_surface.h>
 #include <wlr/render/wlr_texture.h>
 #include <wlr/types/wlr_matrix.h>
 #include <phosh-private-protocol.h>
@@ -174,7 +173,7 @@ phoc_phosh_private_accelerator_already_subscribed (PhocKeyCombo *combo)
   PhocPhoshPrivateKeyboardEventData *kbevent;
   PhocServer *server = phoc_server_get_default ();
 
-  PhocPhoshPrivate *phosh = server->desktop->phosh;
+  PhocPhoshPrivate *phosh = phoc_desktop_get_phosh_private (server->desktop);
 
   for (l = phosh->keyboard_events; l != NULL; l = l->next) {
     kbevent = (PhocPhoshPrivateKeyboardEventData *)l->data;
@@ -443,9 +442,8 @@ thumbnail_frame_handle_copy (struct wl_client   *wl_client,
     zwlr_screencopy_frame_v1_send_failed (frame->resource);
     return;
   }
-  enum zwlr_screencopy_frame_v1_flags flags = (renderer_flags & WLR_RENDERER_READ_PIXELS_Y_INVERT) ? ZWLR_SCREENCOPY_FRAME_V1_FLAGS_Y_INVERT : 0;
 
-  zwlr_screencopy_frame_v1_send_flags (frame->resource, flags);
+  zwlr_screencopy_frame_v1_send_flags (frame->resource, 0);
 
   struct timespec now;
   clock_gettime (CLOCK_MONOTONIC, &now);
@@ -697,8 +695,8 @@ phosh_private_bind (struct wl_client *client, void *data, uint32_t version, uint
 static PhocPhoshPrivate *
 phoc_phosh_private_from_resource (struct wl_resource *resource)
 {
-  assert (wl_resource_instance_of (resource, &phosh_private_interface,
-                                   &phosh_private_impl));
+  g_assert (wl_resource_instance_of (resource, &phosh_private_interface,
+                                     &phosh_private_impl));
   return wl_resource_get_user_data (resource);
 }
 
@@ -706,8 +704,8 @@ phoc_phosh_private_from_resource (struct wl_resource *resource)
 static PhocPhoshPrivateScreencopyFrame *
 phoc_phosh_private_screencopy_frame_from_resource (struct wl_resource *resource)
 {
-  assert (wl_resource_instance_of (resource, &zwlr_screencopy_frame_v1_interface,
-                                   &phoc_phosh_private_screencopy_frame_impl));
+  g_assert (wl_resource_instance_of (resource, &zwlr_screencopy_frame_v1_interface,
+                                     &phoc_phosh_private_screencopy_frame_impl));
   return wl_resource_get_user_data (resource);
 }
 
@@ -715,8 +713,8 @@ phoc_phosh_private_screencopy_frame_from_resource (struct wl_resource *resource)
 static PhocPhoshPrivateKeyboardEventData *
 phoc_phosh_private_keyboard_event_from_resource (struct wl_resource *resource)
 {
-  assert (wl_resource_instance_of (resource, &phosh_private_keyboard_event_interface,
-                                   &phoc_phosh_private_keyboard_event_impl));
+  g_assert (wl_resource_instance_of (resource, &phosh_private_keyboard_event_interface,
+                                     &phoc_phosh_private_keyboard_event_impl));
   return wl_resource_get_user_data (resource);
 }
 
@@ -724,8 +722,8 @@ phoc_phosh_private_keyboard_event_from_resource (struct wl_resource *resource)
 static PhocPhoshPrivateStartupTracker *
 phoc_phosh_private_startup_tracker_from_resource (struct wl_resource *resource)
 {
-  assert (wl_resource_instance_of (resource, &phosh_private_startup_tracker_interface,
-                                   &phoc_phosh_private_startup_tracker_impl));
+  g_assert (wl_resource_instance_of (resource, &phosh_private_startup_tracker_interface,
+                                     &phoc_phosh_private_startup_tracker_impl));
   return wl_resource_get_user_data (resource);
 }
 
@@ -801,7 +799,7 @@ phoc_phosh_private_forward_keysym (PhocKeyCombo *combo,
 {
   GList *l;
   PhocServer *server = phoc_server_get_default ();
-  PhocPhoshPrivate *phosh = server->desktop->phosh;
+  PhocPhoshPrivate *phosh = phoc_desktop_get_phosh_private (server->desktop);
   bool forwarded = false;
 
   for (l = phosh->keyboard_events; l != NULL; l = l->next) {
