@@ -21,14 +21,15 @@
 #include <wlr/types/wlr_idle.h>
 #include <wlr/types/wlr_layer_shell_v1.h>
 #include <wlr/types/wlr_primary_selection.h>
-#include <wlr/types/wlr_switch.h>
 #include <wlr/types/wlr_tablet_v2.h>
 #include <wlr/types/wlr_xcursor_manager.h>
 #include <wlr/types/wlr_tablet_pad.h>
 #include <wlr/version.h>
 #include "cursor.h"
+#include "device-state.h"
 #include "keyboard.h"
 #include "pointer.h"
+#include "switch.h"
 #include "seat.h"
 #include "server.h"
 #include "tablet.h"
@@ -45,7 +46,18 @@ enum {
 };
 static GParamSpec *props[PROP_LAST_PROP];
 
-G_DEFINE_TYPE (PhocSeat, phoc_seat, G_TYPE_OBJECT)
+typedef struct _PhocSeatPrivate {
+  PhocInput             *input;
+  PhocDeviceState       *device_state;
+  char                  *name;
+
+  struct wl_client      *exclusive_client;
+
+
+  GHashTable            *input_mapping_settings;
+} PhocSeatPrivate;
+
+G_DEFINE_TYPE_WITH_PRIVATE (PhocSeat, phoc_seat, G_TYPE_OBJECT)
 
 
 static void
@@ -55,14 +67,15 @@ phoc_seat_set_property (GObject      *object,
                          GParamSpec   *pspec)
 {
   PhocSeat *self = PHOC_SEAT (object);
+  PhocSeatPrivate *priv = phoc_seat_get_instance_private (self);
 
   switch (property_id) {
   case PROP_INPUT:
     /* Don't hold a ref since the input object "owns" the seat */
-    self->input = g_value_get_object (value);
+    priv->input = g_value_get_object (value);
     break;
   case PROP_NAME:
-    self->name = g_value_dup_string (value);
+    priv->name = g_value_dup_string (value);
     break;
   default:
     G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
@@ -78,13 +91,14 @@ phoc_seat_get_property (GObject    *object,
                         GParamSpec *pspec)
 {
   PhocSeat *self = PHOC_SEAT (object);
+  PhocSeatPrivate *priv = phoc_seat_get_instance_private (self);
 
   switch (property_id) {
   case PROP_INPUT:
-    g_value_set_object (value, self->input);
+    g_value_set_object (value, priv->input);
     break;
   case PROP_NAME:
-    g_value_set_string (value, self->name);
+    g_value_set_string (value, priv->name);
     break;
   default:
     G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
@@ -98,7 +112,7 @@ handle_swipe_begin (struct wl_listener *listener, void *data)
   PhocServer *server = phoc_server_get_default ();
   PhocCursor *cursor = wl_container_of (listener, cursor, swipe_begin);
   struct wlr_pointer_gestures_v1 *gestures = server->desktop->pointer_gestures;
-  struct wlr_event_pointer_swipe_begin *event = data;
+  struct wlr_pointer_swipe_begin_event *event = data;
 
   wlr_pointer_gestures_v1_send_swipe_begin (gestures, cursor->seat->seat,
                                             event->time_msec, event->fingers);
@@ -110,7 +124,7 @@ handle_swipe_update (struct wl_listener *listener, void *data)
   PhocServer *server = phoc_server_get_default ();
   PhocCursor *cursor = wl_container_of (listener, cursor, swipe_update);
   struct wlr_pointer_gestures_v1 *gestures = server->desktop->pointer_gestures;
-  struct wlr_event_pointer_swipe_update *event = data;
+  struct wlr_pointer_swipe_update_event *event = data;
 
   wlr_pointer_gestures_v1_send_swipe_update (gestures, cursor->seat->seat,
                                              event->time_msec, event->dx, event->dy);
@@ -122,7 +136,7 @@ handle_swipe_end (struct wl_listener *listener, void *data)
   PhocServer *server = phoc_server_get_default ();
   PhocCursor *cursor = wl_container_of (listener, cursor, swipe_end);
   struct wlr_pointer_gestures_v1 *gestures = server->desktop->pointer_gestures;
-  struct wlr_event_pointer_swipe_end *event = data;
+  struct wlr_pointer_swipe_end_event *event = data;
 
   wlr_pointer_gestures_v1_send_swipe_end (gestures, cursor->seat->seat,
                                           event->time_msec, event->cancelled);
@@ -134,7 +148,7 @@ handle_pinch_begin (struct wl_listener *listener, void *data)
   PhocServer *server = phoc_server_get_default ();
   PhocCursor *cursor = wl_container_of (listener, cursor, pinch_begin);
   struct wlr_pointer_gestures_v1 *gestures = server->desktop->pointer_gestures;
-  struct wlr_event_pointer_pinch_begin *event = data;
+  struct wlr_pointer_pinch_begin_event *event = data;
 
   wlr_pointer_gestures_v1_send_pinch_begin (gestures, cursor->seat->seat,
                                             event->time_msec, event->fingers);
@@ -146,7 +160,7 @@ handle_pinch_update (struct wl_listener *listener, void *data)
   PhocServer *server = phoc_server_get_default ();
   PhocCursor *cursor = wl_container_of (listener, cursor, pinch_update);
   struct wlr_pointer_gestures_v1 *gestures = server->desktop->pointer_gestures;
-  struct wlr_event_pointer_pinch_update *event = data;
+  struct wlr_pointer_pinch_update_event *event = data;
 
   wlr_pointer_gestures_v1_send_pinch_update (gestures, cursor->seat->seat,
                                              event->time_msec, event->dx, event->dy,
@@ -160,24 +174,28 @@ handle_pinch_end (struct wl_listener *listener, void *data)
   PhocCursor *cursor = wl_container_of (listener, cursor, pinch_end);
   struct wlr_pointer_gestures_v1 *gestures =
     server->desktop->pointer_gestures;
-  struct wlr_event_pointer_pinch_end *event = data;
+  struct wlr_pointer_pinch_end_event *event = data;
 
   wlr_pointer_gestures_v1_send_pinch_end (gestures, cursor->seat->seat,
                                           event->time_msec, event->cancelled);
 }
 
 static void
-handle_switch_toggle (struct wl_listener *listener, void *data)
+on_switch_toggled (PhocSeat *self, gboolean state, PhocSwitch *switch_)
 {
   PhocServer *server = phoc_server_get_default ();
-  struct phoc_switch *switch_device =
-    wl_container_of (listener, switch_device, toggle);
   PhocDesktop *desktop = server->desktop;
+  PhocSeatPrivate *priv = phoc_seat_get_instance_private (self);
 
-  wlr_idle_notify_activity (desktop->idle, switch_device->seat->seat);
-  struct wlr_event_switch_toggle *event = data;
+  if (phoc_switch_is_tablet_mode_switch (switch_)) {
+    phoc_device_state_notify_tablet_mode_change (priv->device_state, state);
+  } else if (phoc_switch_is_lid_switch (switch_)) {
+    phoc_device_state_notify_lid_change (priv->device_state, state);
+  } else {
+    g_assert_not_reached ();
+  }
 
-  phoc_switch_handle_toggle (switch_device, event);
+  wlr_idle_notify_activity (desktop->idle, self->seat);
 }
 
 static void
@@ -185,10 +203,10 @@ handle_touch_down (struct wl_listener *listener, void *data)
 {
   PhocServer *server = phoc_server_get_default ();
   PhocCursor *cursor = wl_container_of (listener, cursor, touch_down);
-  struct wlr_event_touch_down *event = data;
+  struct wlr_touch_down_event *event = data;
   PhocDesktop *desktop = server->desktop;
   PhocOutput *output = g_hash_table_lookup (desktop->input_output_map,
-                                            event->device->name);
+                                            event->touch->base.name);
 
   if (output && !output->wlr_output->enabled) {
     g_debug ("Touch event ignored since output '%s' is disabled.",
@@ -204,7 +222,7 @@ handle_touch_up (struct wl_listener *listener, void *data)
 {
   PhocServer *server = phoc_server_get_default ();
   PhocCursor *cursor = wl_container_of (listener, cursor, touch_up);
-  struct wlr_event_touch_up *event = data;
+  struct wlr_touch_up_event *event = data;
   PhocDesktop *desktop = server->desktop;
 
   if (!phoc_cursor_is_active_touch_id (cursor, event->touch_id)) {
@@ -219,7 +237,7 @@ handle_touch_motion (struct wl_listener *listener, void *data)
 {
   PhocServer *server = phoc_server_get_default ();
   PhocCursor *cursor = wl_container_of (listener, cursor, touch_motion);
-  struct wlr_event_touch_motion *event = data;
+  struct wlr_touch_motion_event *event = data;
   PhocDesktop *desktop = server->desktop;
 
   if (!phoc_cursor_is_active_touch_id (cursor, event->touch_id)) {
@@ -285,7 +303,7 @@ handle_tool_axis (struct wl_listener *listener, void *data)
   PhocDesktop *desktop = server->desktop;
 
   wlr_idle_notify_activity (desktop->idle, cursor->seat->seat);
-  struct wlr_event_tablet_tool_axis *event = data;
+  struct wlr_tablet_tool_axis_event *event = data;
   PhocTabletTool *phoc_tool = event->tool->data;
 
   if (!phoc_tool) {       // Should this be an assert?
@@ -297,7 +315,7 @@ handle_tool_axis (struct wl_listener *listener, void *data)
    * We need to handle them ourselves, not pass it into the cursor
    * without any consideration
    */
-  handle_tablet_tool_position (cursor, event->device->data, event->tool,
+  handle_tablet_tool_position (cursor, event->tablet->base.data, event->tool,
                                event->updated_axes & WLR_TABLET_TOOL_AXIS_X,
                                event->updated_axes & WLR_TABLET_TOOL_AXIS_Y,
                                event->x, event->y, event->dx, event->dy);
@@ -350,7 +368,7 @@ handle_tool_tip (struct wl_listener *listener, void *data)
   PhocDesktop *desktop = server->desktop;
 
   wlr_idle_notify_activity (desktop->idle, cursor->seat->seat);
-  struct wlr_event_tablet_tool_tip *event = data;
+  struct wlr_tablet_tool_tip_event *event = data;
   PhocTabletTool *phoc_tool = event->tool->data;
 
   if (event->state == WLR_TABLET_TOOL_TIP_DOWN) {
@@ -384,7 +402,7 @@ handle_tool_button (struct wl_listener *listener, void *data)
   PhocDesktop *desktop = server->desktop;
 
   wlr_idle_notify_activity (desktop->idle, cursor->seat->seat);
-  struct wlr_event_tablet_tool_button *event = data;
+  struct wlr_tablet_tool_button_event *event = data;
   PhocTabletTool *phoc_tool = event->tool->data;
 
   wlr_tablet_v2_tablet_tool_notify_button (phoc_tool->tablet_v2_tool,
@@ -421,7 +439,7 @@ handle_tool_proximity (struct wl_listener *listener, void *data)
   PhocDesktop *desktop = server->desktop;
 
   wlr_idle_notify_activity (desktop->idle, cursor->seat->seat);
-  struct wlr_event_tablet_tool_proximity *event = data;
+  struct wlr_tablet_tool_proximity_event *event = data;
 
   struct wlr_tablet_tool *tool = event->tool;
 
@@ -455,7 +473,7 @@ handle_tool_proximity (struct wl_listener *listener, void *data)
     return;
   }
 
-  handle_tablet_tool_position (cursor, event->device->data, event->tool,
+  handle_tablet_tool_position (cursor, event->tablet->base.data, event->tool,
                                true, true, event->x, event->y, 0, 0);
 }
 
@@ -483,12 +501,13 @@ handle_pointer_focus_change (struct wl_listener *listener,
 static PhocOutput *
 get_output_from_settings (PhocSeat *self, PhocInputDevice *device)
 {
+  PhocSeatPrivate *priv = phoc_seat_get_instance_private (self);
   PhocServer *server = phoc_server_get_default ();
   PhocDesktop *desktop = server->desktop;
   GSettings *settings;
   g_auto (GStrv) edid = NULL;
 
-  settings = g_hash_table_lookup (self->input_mapping_settings, device);
+  settings = g_hash_table_lookup (priv->input_mapping_settings, device);
   g_assert (G_IS_SETTINGS (settings));
 
   edid = g_settings_get_strv (settings, "output");
@@ -688,7 +707,7 @@ phoc_drag_icon_handle_destroy (struct wl_listener *listener,
 
   phoc_drag_icon_damage_whole (icon);
 
-  assert (icon->seat->drag_icon == icon);
+  g_assert (icon->seat->drag_icon == icon);
   icon->seat->drag_icon = NULL;
 
   wl_list_remove (&icon->surface_commit.link);
@@ -750,7 +769,7 @@ phoc_seat_handle_start_drag (struct wl_listener *listener,
   icon->destroy.notify = phoc_drag_icon_handle_destroy;
   wl_signal_add (&wlr_drag_icon->events.destroy, &icon->destroy);
 
-  assert (seat->drag_icon == NULL);
+  g_assert (seat->drag_icon == NULL);
   seat->drag_icon = icon;
 
   phoc_drag_icon_update_position (icon);
@@ -786,7 +805,7 @@ phoc_drag_icon_update_position (PhocDragIcon *icon)
   PhocSeat *seat = icon->seat;
   struct wlr_drag *wlr_drag = icon->wlr_drag_icon->drag;
 
-  assert (wlr_drag != NULL);
+  g_assert (wlr_drag != NULL);
 
   switch (seat->seat->drag->grab_type) {
   case WLR_DRAG_GRAB_KEYBOARD_POINTER:;
@@ -845,22 +864,25 @@ phoc_seat_handle_destroy (struct wl_listener *listener,
 
 
 static void
-seat_update_capabilities (PhocSeat *seat)
+seat_update_capabilities (PhocSeat *self)
 {
+  PhocSeatPrivate *priv = phoc_seat_get_instance_private (self);
   uint32_t caps = 0;
 
-  if (seat->keyboards != NULL) {
+  if (self->keyboards != NULL) {
     caps |= WL_SEAT_CAPABILITY_KEYBOARD;
   }
-  if (seat->pointers != NULL) {
+  if (self->pointers != NULL) {
     caps |= WL_SEAT_CAPABILITY_POINTER;
   }
-  if (seat->touch != NULL) {
+  if (self->touch != NULL) {
     caps |= WL_SEAT_CAPABILITY_TOUCH;
   }
-  wlr_seat_set_capabilities (seat->seat, caps);
+  wlr_seat_set_capabilities (self->seat, caps);
 
-  phoc_seat_maybe_set_cursor (seat, seat->cursor->default_xcursor);
+  phoc_seat_maybe_set_cursor (self, self->cursor->default_xcursor);
+
+  phoc_device_state_update_capabilities (priv->device_state);
 }
 
 
@@ -877,6 +899,7 @@ on_settings_output_changed (PhocSeat *seat)
 static void
 phoc_seat_add_input_mapping_settings (PhocSeat *self, PhocInputDevice *device)
 {
+  PhocSeatPrivate *priv = phoc_seat_get_instance_private (self);
   const char *schema, *group, *vendor, *product;
   g_autofree char *path = NULL;
   g_autoptr (GSettings) settings = NULL;
@@ -903,7 +926,7 @@ phoc_seat_add_input_mapping_settings (PhocSeat *self, PhocInputDevice *device)
   settings = g_settings_new_with_path (schema, path);
   g_signal_connect_swapped (settings, "changed::output",
                             G_CALLBACK (on_settings_output_changed), self);
-  g_hash_table_insert (self->input_mapping_settings, device, g_steal_pointer (&settings));
+  g_hash_table_insert (priv->input_mapping_settings, device, g_steal_pointer (&settings));
   on_settings_output_changed (self);
 }
 
@@ -937,7 +960,7 @@ static void
 seat_add_keyboard (PhocSeat                *seat,
                    struct wlr_input_device *device)
 {
-  assert (device->type == WLR_INPUT_DEVICE_KEYBOARD);
+  g_assert (device->type == WLR_INPUT_DEVICE_KEYBOARD);
   PhocKeyboard *keyboard = phoc_keyboard_new (device, seat);
 
   seat->keyboards = g_slist_prepend (seat->keyboards, keyboard);
@@ -950,7 +973,7 @@ seat_add_keyboard (PhocSeat                *seat,
                             G_CALLBACK (on_keyboard_activity),
                             seat);
 
-  wlr_seat_set_keyboard (seat->seat, device);
+  wlr_seat_set_keyboard (seat->seat, wlr_keyboard_from_input_device (device));
 }
 
 static void
@@ -983,40 +1006,41 @@ seat_add_pointer (PhocSeat                *seat,
 }
 
 static void
-handle_switch_destroy (struct wl_listener *listener, void *data)
+on_switch_destroy (PhocSwitch *switch_)
 {
-  struct phoc_switch *switch_device =
-    wl_container_of (listener, switch_device, device_destroy);
-  PhocSeat *seat = switch_device->seat;
+  PhocSeat *seat = phoc_input_device_get_seat (PHOC_INPUT_DEVICE (switch_));
 
-  wl_list_remove (&switch_device->link);
-  wl_list_remove (&switch_device->device_destroy.link);
-  free (switch_device);
+  g_assert (PHOC_IS_SWITCH (switch_));
+  g_debug ("Removing switch device: %s", phoc_input_device_get_name (PHOC_INPUT_DEVICE (switch_)));
+  seat->switches = g_slist_remove (seat->switches, switch_);
+  g_object_unref (switch_);
 
   seat_update_capabilities (seat);
 }
 
 static void
-seat_add_switch (PhocSeat                *seat,
-                 struct wlr_input_device *device)
+seat_add_switch (PhocSeat *self, struct wlr_input_device *device)
 {
-  assert (device->type == WLR_INPUT_DEVICE_SWITCH);
-  struct phoc_switch *switch_device = g_new0 (struct phoc_switch, 1);
+  PhocSwitch *switch_ = phoc_switch_new (device, self);
 
-  device->data = switch_device;
-  switch_device->device = device;
-  switch_device->seat = seat;
-  wl_list_insert (&seat->switches, &switch_device->link);
-  switch_device->device_destroy.notify = handle_switch_destroy;
+  self->switches = g_slist_prepend (self->switches, switch_);
+  g_signal_connect (switch_, "device-destroy",
+                    G_CALLBACK (on_switch_destroy),
+                    NULL);
 
-  switch_device->toggle.notify = handle_switch_toggle;
-  wl_signal_add (&switch_device->device->switch_device->events.toggle, &switch_device->toggle);
+  g_signal_connect_object (switch_, "toggled",
+                           G_CALLBACK (on_switch_toggled),
+                           self,
+                           G_CONNECT_SWAPPED);
+
+  seat_update_capabilities (self);
 }
 
 static void
 on_touch_destroy (PhocTouch *touch)
 {
   PhocSeat *seat = phoc_input_device_get_seat (PHOC_INPUT_DEVICE (touch));
+  PhocSeatPrivate *priv = phoc_seat_get_instance_private (seat);
   PhocServer *server = phoc_server_get_default ();
   PhocDesktop *desktop = server->desktop;
   struct wlr_input_device *device = phoc_input_device_get_device (PHOC_INPUT_DEVICE (touch));
@@ -1024,7 +1048,7 @@ on_touch_destroy (PhocTouch *touch)
   g_assert (PHOC_IS_TOUCH (touch));
   g_debug ("Removing touch device: %s", device->name);
   g_hash_table_remove (desktop->input_output_map, device->name);
-  g_hash_table_remove (seat->input_mapping_settings, touch);
+  g_hash_table_remove (priv->input_mapping_settings, touch);
 
   seat->touch = g_slist_remove (seat->touch, touch);
   wlr_cursor_detach_input_device (seat->cursor->cursor, device);
@@ -1113,7 +1137,7 @@ handle_tablet_pad_ring (struct wl_listener *listener, void *data)
 {
   PhocTabletPad *pad =
     wl_container_of (listener, pad, ring);
-  struct wlr_event_tablet_pad_ring *event = data;
+  struct wlr_tablet_pad_ring_event *event = data;
 
   wlr_tablet_v2_tablet_pad_notify_ring (pad->tablet_v2_pad,
                                         event->ring, event->position,
@@ -1126,7 +1150,7 @@ handle_tablet_pad_strip (struct wl_listener *listener, void *data)
 {
   PhocTabletPad *pad =
     wl_container_of (listener, pad, strip);
-  struct wlr_event_tablet_pad_strip *event = data;
+  struct wlr_tablet_pad_strip_event *event = data;
 
   wlr_tablet_v2_tablet_pad_notify_strip (pad->tablet_v2_pad,
                                          event->strip, event->position,
@@ -1139,7 +1163,7 @@ handle_tablet_pad_button (struct wl_listener *listener, void *data)
 {
   PhocTabletPad *pad =
     wl_container_of (listener, pad, button);
-  struct wlr_event_tablet_pad_button *event = data;
+  struct wlr_tablet_pad_button_event *event = data;
 
   wlr_tablet_v2_tablet_pad_notify_mode (pad->tablet_v2_pad,
                                         event->group, event->mode, event->time_msec);
@@ -1155,6 +1179,7 @@ seat_add_tablet_pad (PhocSeat                *seat,
 {
   PhocServer *server = phoc_server_get_default ();
   PhocTabletPad *tablet_pad = g_new0 (PhocTabletPad, 1);
+  struct wlr_tablet_pad *wlr_tablet_pad = wlr_tablet_pad_from_input_device (device);
 
   device->data = tablet_pad;
   tablet_pad->device = device;
@@ -1166,17 +1191,17 @@ seat_add_tablet_pad (PhocSeat                *seat,
                  &tablet_pad->device_destroy);
 
   tablet_pad->attach.notify = handle_tablet_pad_attach;
-  wl_signal_add (&tablet_pad->device->tablet_pad->events.attach_tablet,
+  wl_signal_add (&wlr_tablet_pad->events.attach_tablet,
                  &tablet_pad->attach);
 
   tablet_pad->button.notify = handle_tablet_pad_button;
-  wl_signal_add (&tablet_pad->device->tablet_pad->events.button, &tablet_pad->button);
+  wl_signal_add (&wlr_tablet_pad->events.button, &tablet_pad->button);
 
   tablet_pad->strip.notify = handle_tablet_pad_strip;
-  wl_signal_add (&tablet_pad->device->tablet_pad->events.strip, &tablet_pad->strip);
+  wl_signal_add (&wlr_tablet_pad->events.strip, &tablet_pad->strip);
 
   tablet_pad->ring.notify = handle_tablet_pad_ring;
-  wl_signal_add (&tablet_pad->device->tablet_pad->events.ring, &tablet_pad->ring);
+  wl_signal_add (&wlr_tablet_pad->events.ring, &tablet_pad->ring);
 
   wl_list_init (&tablet_pad->tablet_destroy.link);
 
@@ -1212,12 +1237,15 @@ seat_add_tablet_pad (PhocSeat                *seat,
 static void
 on_tablet_destroy (PhocSeat *seat, PhocTablet *tablet)
 {
+  PhocSeatPrivate *priv = phoc_seat_get_instance_private (seat);
   PhocServer *server = phoc_server_get_default ();
   PhocDesktop *desktop = server->desktop;
   struct wlr_input_device *device = phoc_input_device_get_device (PHOC_INPUT_DEVICE (tablet));
 
+  g_assert (PHOC_IS_TABLET (tablet));
+  g_debug ("Removing tablet device: %s", device->name);
   wlr_cursor_detach_input_device (seat->cursor->cursor, device);
-  g_hash_table_remove (seat->input_mapping_settings, tablet);
+  g_hash_table_remove (priv->input_mapping_settings, tablet);
   g_hash_table_remove (desktop->input_output_map, device->name);
 
   seat->tablets = g_slist_remove (seat->tablets, tablet);
@@ -1326,7 +1354,7 @@ phoc_seat_has_meta_pressed (PhocSeat *seat)
     struct wlr_input_device *device = phoc_input_device_get_device (input_device);
 
     uint32_t modifiers =
-      wlr_keyboard_get_modifiers (device->keyboard);
+      wlr_keyboard_get_modifiers (wlr_keyboard_from_input_device(device));
     if ((modifiers ^ phoc_keyboard_get_meta_key (keyboard)) == 0) {
       return true;
     }
@@ -1337,13 +1365,13 @@ phoc_seat_has_meta_pressed (PhocSeat *seat)
 
 
 /**
- * phoc_seat_get_focus:
+ * phoc_seat_get_focus_view:
  * @seat: The seat
  *
  * Returns: (nullable)(transfer none): The currently focused view
  */
 PhocView *
-phoc_seat_get_focus (PhocSeat *seat)
+phoc_seat_get_focus_view (PhocSeat *seat)
 {
   if (!seat->has_focus || wl_list_empty (&seat->views)) {
     return NULL;
@@ -1363,7 +1391,7 @@ seat_view_destroy (PhocSeatView *seat_view)
   g_assert (PHOC_IS_SEAT (seat));
   g_assert (PHOC_IS_VIEW (view));
 
-  if (view == phoc_seat_get_focus (seat)) {
+  if (view == phoc_seat_get_focus_view (seat)) {
     seat->has_focus = false;
     seat->cursor->mode = PHOC_CURSOR_PASSTHROUGH;
   }
@@ -1377,12 +1405,12 @@ seat_view_destroy (PhocSeatView *seat_view)
   free (seat_view);
 
   if (view && view->parent) {
-    phoc_seat_set_focus (seat, view->parent);
+    phoc_seat_set_focus_view (seat, view->parent);
   } else if (!wl_list_empty (&seat->views)) {
     // Focus first view
     PhocSeatView *first_seat_view = wl_container_of (
       seat->views.next, first_seat_view, link);
-    phoc_seat_set_focus (seat, first_seat_view->view);
+    phoc_seat_set_focus_view (seat, first_seat_view->view);
   }
 }
 
@@ -1448,14 +1476,20 @@ phoc_seat_view_from_view (PhocSeat *seat, PhocView *view)
   return seat_view;
 }
 
+
 bool
 phoc_seat_allow_input (PhocSeat           *seat,
                        struct wl_resource *resource)
 {
-  return !seat->exclusive_client ||
-         wl_resource_get_client (resource) == seat->exclusive_client;
+  PhocSeatPrivate *priv;
+
+  g_assert (PHOC_IS_SEAT (seat));
+  priv = phoc_seat_get_instance_private (seat);
+
+  return !priv->exclusive_client || wl_resource_get_client (resource) == priv->exclusive_client;
 }
 
+
 static void
 seat_raise_view_stack (PhocSeat *seat, PhocView *view)
 {
@@ -1477,9 +1511,23 @@ seat_raise_view_stack (PhocSeat *seat, PhocView *view)
   }
 }
 
+/**
+ * phoc_seat_set_focus_view:
+ * @seat: The seat
+ * @view:(nullable): The view to focus
+ *
+ * If possible it will unfocus the currently focused view and focus
+ * the given %view, raise it if necessary and make it appear
+ * activated. If %NULL is passed only the current view is unfocsed.
+ */
 void
-phoc_seat_set_focus (PhocSeat *seat, PhocView *view)
+phoc_seat_set_focus_view (PhocSeat *seat, PhocView *view)
 {
+  PhocSeatPrivate *priv;
+
+  g_assert (PHOC_IS_SEAT (seat));
+  priv = phoc_seat_get_instance_private (seat);
+
   if (view && !phoc_seat_allow_input (seat, view->wlr_surface->resource)) {
     return;
   }
@@ -1526,8 +1574,7 @@ phoc_seat_set_focus (PhocSeat *seat, PhocView *view)
     }
   }
 
-  PhocView *prev_focus = phoc_seat_get_focus (seat);
-
+  PhocView *prev_focus = phoc_seat_get_focus_view (seat);
   if (view && view == prev_focus) {
     return;
   }
@@ -1552,7 +1599,7 @@ phoc_seat_set_focus (PhocSeat *seat, PhocView *view)
   seat->has_focus = false;
 
   // Deactivate the old view if it is not focused by some other seat
-  if (prev_focus != NULL && !phoc_input_view_has_focus (seat->input, prev_focus)) {
+  if (prev_focus != NULL && !phoc_input_view_has_focus (priv->input, prev_focus)) {
     phoc_view_activate (prev_focus, false);
   }
 
@@ -1563,9 +1610,14 @@ phoc_seat_set_focus (PhocSeat *seat, PhocView *view)
     return;
   }
 
+  /* Next view to receive focus */
   wl_list_remove (&seat_view->link);
   wl_list_insert (&seat->views, &seat_view->link);
 
+  /* Flush the token early as a layer surface might have focus */
+  if (phoc_view_get_activation_token (view))
+    phoc_view_flush_activation_token (view);
+
   if (seat->focused_layer) {
     return;
   }
@@ -1614,22 +1666,26 @@ phoc_seat_set_focus_layer (PhocSeat                    *seat,
         // Focus first view
         PhocSeatView *first_seat_view = wl_container_of (
           seat->views.next, first_seat_view, link);
-        phoc_seat_set_focus (seat, first_seat_view->view);
+        phoc_seat_set_focus_view (seat, first_seat_view->view);
       } else {
-        phoc_seat_set_focus (seat, NULL);
+        phoc_seat_set_focus_view (seat, NULL);
       }
       phoc_layer_shell_arrange (output);
       phoc_output_update_shell_reveal (output);
     }
     return;
   }
+
+  // An existing keyboard grab might try to deny setting focus, so cancel it
+  wlr_seat_keyboard_end_grab (seat->seat);
+
   struct wlr_keyboard *keyboard = wlr_seat_get_keyboard (seat->seat);
 
   if (!phoc_seat_allow_input (seat, layer->resource)) {
     return;
   }
   if (seat->has_focus) {
-    PhocView *prev_focus = phoc_seat_get_focus (seat);
+    PhocView *prev_focus = phoc_seat_get_focus_view (seat);
     wlr_seat_keyboard_clear_focus (seat->seat);
     phoc_view_activate (prev_focus, false);
   }
@@ -1651,12 +1707,23 @@ phoc_seat_set_focus_layer (PhocSeat                    *seat,
   phoc_output_update_shell_reveal (PHOC_OUTPUT (layer->output->data));
 }
 
+/**
+ * phoc_seat_set_exclusive_client:
+ * @seat: The seat
+ * @client:(nullable): The exclusive client
+ *
+ * If %client is no %NULL only this client can receive input events.
+ */
 void
-phoc_seat_set_exclusive_client (PhocSeat         *seat,
-                                struct wl_client *client)
+phoc_seat_set_exclusive_client (PhocSeat *seat, struct wl_client *client)
 {
+  PhocSeatPrivate *priv;
+
+  g_assert (PHOC_IS_SEAT (seat));
+  priv = phoc_seat_get_instance_private (seat);
+
   if (!client) {
-    seat->exclusive_client = client;
+    priv->exclusive_client = client;
     // Triggers a refocus of the topmost surface layer if necessary
     phoc_layer_shell_update_focus ();
     return;
@@ -1667,9 +1734,9 @@ phoc_seat_set_exclusive_client (PhocSeat         *seat,
     }
   }
   if (seat->has_focus) {
-    PhocView *focus = phoc_seat_get_focus (seat);
+    PhocView *focus = phoc_seat_get_focus_view (seat);
     if (wl_resource_get_client (focus->wlr_surface->resource) != client) {
-      phoc_seat_set_focus (seat, NULL);
+      phoc_seat_set_focus_view (seat, NULL);
     }
   }
   if (seat->seat->pointer_state.focused_client) {
@@ -1688,7 +1755,7 @@ phoc_seat_set_exclusive_client (PhocSeat         *seat,
                                         now.tv_nsec / 1000, point->touch_id);
     }
   }
-  seat->exclusive_client = client;
+  priv->exclusive_client = client;
 }
 
 /**
@@ -1710,7 +1777,7 @@ phoc_seat_cycle_focus (PhocSeat *seat, gboolean forward)
   first_seat_view = wl_container_of (seat->views.next, first_seat_view, link);
 
   if (!seat->has_focus) {
-    phoc_seat_set_focus (seat, first_seat_view->view);
+    phoc_seat_set_focus_view (seat, first_seat_view->view);
     return;
   }
 
@@ -1726,7 +1793,7 @@ phoc_seat_cycle_focus (PhocSeat *seat, gboolean forward)
   }
 
   g_assert (PHOC_IS_VIEW (seat_view->view));
-  phoc_seat_set_focus (seat, seat_view->view);
+  phoc_seat_set_focus_view (seat, seat_view->view);
 
   if (forward) {
     /* Move the first view to the end */
@@ -1820,7 +1887,7 @@ void
 phoc_seat_end_compositor_grab (PhocSeat *seat)
 {
   PhocCursor *cursor = seat->cursor;
-  PhocView *view = phoc_seat_get_focus (seat);
+  PhocView *view = phoc_seat_get_focus_view (seat);
 
   if (view == NULL) {
     return;
@@ -1889,10 +1956,11 @@ phoc_seat_constructed (GObject *object)
 {
   PhocSeat *self = PHOC_SEAT(object);
   PhocServer *server = phoc_server_get_default ();
+  PhocSeatPrivate *priv = phoc_seat_get_instance_private (self);
 
   G_OBJECT_CLASS (phoc_seat_parent_class)->constructed (object);
 
-  self->seat = wlr_seat_create (server->wl_display, self->name);
+  self->seat = wlr_seat_create (server->wl_display, priv->name);
   g_assert (self->seat);
   self->seat->data = self;
 
@@ -1916,6 +1984,8 @@ phoc_seat_constructed (GObject *object)
   wl_signal_add (&self->seat->events.start_drag, &self->start_drag);
   self->destroy.notify = phoc_seat_handle_destroy;
   wl_signal_add (&self->seat->events.destroy, &self->destroy);
+
+  priv->device_state = phoc_device_state_new (self);
 }
 
 
@@ -1923,7 +1993,9 @@ static void
 phoc_seat_dispose (GObject *object)
 {
   PhocSeat *self = PHOC_SEAT (object);
+  PhocSeatPrivate *priv = phoc_seat_get_instance_private (self);
 
+  g_clear_object (&priv->device_state);
   g_clear_object (&self->cursor);
 
   G_OBJECT_CLASS (phoc_seat_parent_class)->dispose (object);
@@ -1934,11 +2006,18 @@ static void
 phoc_seat_finalize (GObject *object)
 {
   PhocSeat *self = PHOC_SEAT (object);
+  PhocSeatPrivate *priv = phoc_seat_get_instance_private (self);
 
-  g_clear_pointer (&self->input_mapping_settings, g_hash_table_destroy);
+  g_clear_pointer (&priv->input_mapping_settings, g_hash_table_destroy);
   phoc_seat_handle_destroy (&self->destroy, self->seat);
   wlr_seat_destroy (self->seat);
-  g_clear_pointer (&self->name, g_free);
+  g_clear_pointer (&priv->name, g_free);
+
+  g_clear_pointer (&self->keyboards, g_slist_free);
+  g_clear_pointer (&self->pointers, g_slist_free);
+  g_clear_pointer (&self->switches, g_slist_free);
+  g_clear_pointer (&self->touch, g_slist_free);
+  g_clear_pointer (&self->tablets, g_slist_free);
 
   G_OBJECT_CLASS (phoc_seat_parent_class)->finalize (object);
 }
@@ -1979,13 +2058,14 @@ phoc_seat_class_init (PhocSeatClass *klass)
 static void
 phoc_seat_init (PhocSeat *self)
 {
+  PhocSeatPrivate *priv = phoc_seat_get_instance_private (self);
+
   wl_list_init (&self->tablet_pads);
-  wl_list_init (&self->switches);
   wl_list_init (&self->views);
 
   self->touch_id = -1;
 
-  self->input_mapping_settings = g_hash_table_new_full (g_direct_hash,
+  priv->input_mapping_settings = g_hash_table_new_full (g_direct_hash,
                                                         g_direct_equal,
                                                         NULL,
                                                         g_object_unref);
@@ -2030,3 +2110,30 @@ phoc_seat_has_keyboard (PhocSeat *self)
   g_assert (self->seat);
   return (self->seat->capabilities & WL_SEAT_CAPABILITY_KEYBOARD);
 }
+
+
+gboolean
+phoc_seat_has_switch (PhocSeat *self, enum wlr_switch_type type)
+{
+  g_assert (PHOC_IS_SEAT (self));
+
+  for (GSList *l = self->switches; l; l = l->next) {
+    PhocSwitch *switch_ = PHOC_SWITCH (l->data);
+
+    if (phoc_switch_is_type (switch_, type))
+      return TRUE;
+  }
+  return FALSE;
+}
+
+/**
+ * phoc_seat_from_wlr_seat:
+ * @wlr_seat: The wlr_seat
+ *
+ * Returns: (transfer none): The [class@Seat] associated with the given wlr_seat
+ */
+PhocSeat *
+phoc_seat_from_wlr_seat (struct wlr_seat *wlr_seat)
+{
+  return PHOC_SEAT (wlr_seat->data);
+}
