@@ -5,7 +5,6 @@
 
 #include <xf86drmMode.h>
 #include <wlr/types/wlr_input_device.h>
-#include <wlr/types/wlr_switch.h>
 #include <wlr/types/wlr_output_layout.h>
 
 G_BEGIN_DECLS
@@ -23,7 +22,7 @@ typedef struct _PhocOutputConfig {
   int                      x, y;
   float                    scale;
 
-  struct Mode {
+  struct PhocMode {
     int   width, height;
     float refresh_rate;
   } mode;
