@@ -115,16 +115,18 @@ apply_size_constraints (PhocView                    *view,
   if (view_is_maximized (view))
     return;
 
-  struct wlr_xwayland_surface_size_hints *size_hints = xwayland_surface->size_hints;
+  xcb_size_hints_t *size_hints = xwayland_surface->size_hints;
   if (size_hints != NULL) {
     if (size_hints->min_width > 0 && width < (uint32_t)size_hints->min_width) {
       *dest_width = size_hints->min_width;
-    } else if (size_hints->max_width > 0 && width > (uint32_t)size_hints->max_width) {
+    } else if (size_hints->max_width > 0 &&
+               width > (uint32_t)size_hints->max_width) {
       *dest_width = size_hints->max_width;
     }
     if (size_hints->min_height > 0 && height < (uint32_t)size_hints->min_height) {
       *dest_height = size_hints->min_height;
-    } else if (size_hints->max_height > 0 && height > (uint32_t)size_hints->max_height) {
+    } else if (size_hints->max_height > 0 &&
+               height > (uint32_t)size_hints->max_height) {
       *dest_height = size_hints->max_height;
     }
   }
@@ -235,6 +237,16 @@ set_fullscreen(PhocView *view, bool fullscreen)
   wlr_xwayland_surface_set_fullscreen (xwayland_surface, fullscreen);
 }
 
+
+static pid_t
+get_pid (PhocView *view)
+{
+  PhocXWaylandSurface *self = PHOC_XWAYLAND_SURFACE (view);
+
+  return self->xwayland_surface->pid;
+}
+
+
 static void
 phoc_xwayland_surface_set_property (GObject      *object,
                                     guint         property_id,
@@ -334,7 +346,7 @@ handle_request_maximize (struct wl_listener *listener, void *data)
   bool maximized = xwayland_surface->maximized_vert && xwayland_surface->maximized_horz;
 
   if (maximized)
-    view_maximize (view, NULL);
+    phoc_view_maximize (view, NULL);
   else
     view_restore (view);
 }
@@ -369,13 +381,21 @@ handle_set_class (struct wl_listener *listener, void *data)
 static void
 handle_set_startup_id (struct wl_listener *listener, void *data)
 {
-  PhocServer *server = phoc_server_get_default ();
   PhocXWaylandSurface *self = wl_container_of (listener, self, set_startup_id);
+  const char *token;
 
   g_debug ("Got startup-id %s", self->xwayland_surface->startup_id);
-  phoc_phosh_private_notify_startup_id (server->desktop->phosh,
-                                        self->xwayland_surface->startup_id,
-                                        PHOSH_PRIVATE_STARTUP_TRACKER_PROTOCOL_X11);
+
+  token = self->xwayland_surface->startup_id;
+  phoc_view_set_activation_token (PHOC_VIEW (self), token, PHOSH_PRIVATE_STARTUP_TRACKER_PROTOCOL_X11);
+  if (phoc_view_is_mapped (PHOC_VIEW (self))) {
+    PhocSeat *seat = phoc_server_get_last_active_seat (phoc_server_get_default ());
+
+    g_debug ("Activating view %p via token '%s'", PHOC_VIEW (self), token);
+    phoc_seat_set_focus_view (seat, PHOC_VIEW (self));
+  } else {
+    g_debug ("Setting view %p via token '%s' as pending activation", PHOC_VIEW (self),token);
+  }
 }
 
 static void
@@ -431,7 +451,7 @@ handle_map (struct wl_listener *listener, void *data)
   wl_signal_add (&surface->surface->events.commit, &self->surface_commit);
 
   if (surface->maximized_horz && surface->maximized_vert)
-    view_maximize (view, NULL);
+    phoc_view_maximize (view, NULL);
 
   view_auto_maximize (view);
 
@@ -440,9 +460,9 @@ handle_map (struct wl_listener *listener, void *data)
   if (!surface->override_redirect) {
     if (surface->decorations == WLR_XWAYLAND_SURFACE_DECORATIONS_ALL)
       phoc_view_set_decoration (view, TRUE, 12, 4);
-    view_setup (view);
+    phoc_view_setup (view);
   } else {
-    view_initial_focus (view);
+    phoc_view_set_initial_focus (view);
   }
 }
 
@@ -555,6 +575,7 @@ phoc_xwayland_surface_class_init (PhocXWaylandSurfaceClass *klass)
   view_class->set_fullscreen = set_fullscreen;
   view_class->set_maximized = set_maximized;
   view_class->close = _close;
+  view_class->get_pid = get_pid;
 
   /**
    * PhocXWaylandSurface:wlr-xwayland-surface:
